// NF Client class implementation
#include "pch.h"

#include "Client.h"
#include "util.h"

namespace nf::client {
    Client::Client(ClientConfig config)
        : m_running(true)
        , m_config(config)
        , m_renderEngine()
    {}

    void Client::run() {
        // Create window on input thread and pass up a pointer for the renderer
        std::promise<std::shared_ptr<Window>> windowPromise;
        auto windowFuture = windowPromise.get_future();
        std::thread inputThread(&Client::runInputThread, this, std::move(windowPromise));
        m_renderEngine = std::make_unique<render::RenderEngine>(std::move(windowFuture.get()), m_config.display);

        while (m_running) {
            m_renderEngine->doFrame();
        }

        inputThread.join();
    }

    void Client::runInputThread(std::promise<std::shared_ptr<Window>> windowPromise) {
#ifdef _DEBUG
        SetThreadDescription(GetCurrentThread(), L"NF Input Thread");
#endif

        std::shared_ptr<Window> window = std::make_shared<Window>(m_config.appName);
        windowPromise.set_value(window);

        window->runLoop();
        m_running = false;
    }
}
