// RenderEngine class header
#pragma once

#include "client/Window.h"
#include "nf/config.h"

namespace nf::client::render {
    struct FrameExecutor {
        VkCommandBuffer commandBuffer;
        VkSemaphore semaphoreImageAvailable, semaphoreRenderFinished;
        VkFence fenceFrameInFlight;
    };

    class RenderEngine {
    public:
        RenderEngine(std::shared_ptr<Window> window, DisplayConfig display);

        void doFrame();

        ~RenderEngine();
    private:
        void createInstance();
        void createSurface();
        void pickPhysicalDevice();
        void createDevice();
        void createSwapchain();
        void createOutputRenderPass();
        void createOutputPipeline();
        void createFrameExecutors();

        std::shared_ptr<Window> m_window;
        DisplayConfig m_display;

        // Global state
        VkInstance m_instance;
        VkSurfaceKHR m_surface;
        VkPhysicalDevice m_physicalDevice;
        uint32_t m_queueFIGraphics, m_queueFIPresent;
        VkDevice m_device;
        VkQueue m_queueGraphics, m_queuePresent;
        VkRenderPass m_renderPassOutput;

        // Swapchain
        VkSwapchainKHR m_swapchain;
        VkFormat m_swapchainImageFormat;
        std::vector<VkImage> m_swapchainImages;
        std::vector<VkImageView> m_swapchainImageViews;
        std::vector<VkFramebuffer> m_swapchainFramebuffers;

        // Pipelines
        VkPipelineLayout m_pipelineLayoutOutput;
        VkPipeline m_pipelineOutput;

        // Execution objects
        VkCommandPool m_commandPool;
        const uint32_t m_numFramesInFlight;
        std::vector<FrameExecutor> m_frameExecutors;
        uint32_t m_currentFrameExecutor;
    };
}
